
/*******************************************************************************

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

 Copyright (C) 2011 Nuno Fortes(nunofort@gmail.com)

FILE: Parser.hpp

DESC: File Parser class

********************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <stdlib.h>
#include <string.h>


class Parser
{
public:
        Parser();
        int real_end(FILE *fptr);
        char *next_keyword(FILE *fptr, char *keyword, char divisor);
        char *read_config(FILE *fptr, char *keyword, char opera);
        void get_line(FILE *fptr,char *buffer, int size);
        char *trim(const char *text);

        char *get_token(const char *msg, char ascii, int num);
        void insert_str(char *s, const char *k, const char *text);
        int found_str(char **from, int max, char *s);
        void clean_str(char *text);
        int is_numeric(const char *text);

private:
        bool is_newline(FILE *fptr);
        void escape_spaces(FILE *fptr);


};

Parser::Parser() {}
	
void Parser::escape_spaces(FILE *fptr)
{
   char c = ' ';
   while (((c == ' ') || (c == '\t') || 
	   (c == '\r') || (c == '\n')) && (!feof(fptr)))
   {
	fread(&c,1,1,fptr);
   }
   fseek(fptr, -1, SEEK_CUR);
}	

bool Parser::is_newline(FILE *fptr)
{
   char c;
   long pos = 0;
   do
   {
	fread(&c,1,1,fptr);
	pos++;
   }
   while (((c == ' ') || (c == '\t') || (c == '\r')) && (!feof(fptr)));
   fseek(fptr, -pos, SEEK_CUR);
   return (c == '\n');
}



char *Parser::trim(const char *text)
{
   char tmp[512];
   int i = 0, ii = 0;
   int len, start, end;
   while ((text[i] == ' ') || (text[i] == '\r') || (text[i] == '\n')) 
	i++;
   start = i;
   for (i = 0; text[i] != '\0'; i++)
	len = i;
   i = len-1;
   while ((text[i] == ' ') || (text[i] == '\r') || (text[i] == '\n')) 
	i--;
   end = i;
   for (ii = 0, i = start; i <= end; i++,ii++)
	tmp[ii] = text[i];
   tmp[ii] = '\0';
   //printf("[%s]\n",tmp);
   return strdup(tmp);
}

int Parser::real_end(FILE *fptr)
{
  char c = ' ';
  long pos = ftell(fptr);
  while (((c == ' ') || (c == '\r') || (c == '\n')) && (!feof(fptr)))
        fread(&c, 1, 1, fptr);
  if (feof(fptr))
        return 1;
  else
  {
        fseek(fptr,pos,SEEK_SET);
        return 0;
  }
}

char *Parser::next_keyword(FILE *fptr, char *keyword, char divisor)
{
    char value[512];
    char c;
    int i;
    do
    {
        fread(&c, 1, 1, fptr);
    }
    while ((c != divisor) && (!feof(fptr)));
    do
    {
        fseek(fptr, -2, SEEK_CUR);
        fread(&c, 1, 1, fptr);
    }
    while ((c != '\n') && (ftell(fptr) > 0));
    i = 0;
    do
    {
        fread(&c, 1, 1, fptr);
        if (c != ' ')
        {
                keyword[i] = c;
                i++;
                if (c == divisor) i--;
        }
    }
    while ((c != divisor) && (!feof(fptr)));
    keyword[i] = '\0';
    i = 0;
    do
    {
        fread(&c, 1, 1, fptr);
        value[i] = c;
        i++;
        if (c == '\n') i--;
    }
    while ((c != '\n') && (!feof(fptr)));
    value[i] = '\0';
    //fscanf(fptr,"%s : %s\n",key,value);
    //printf("%s : %s\n",keyword,value);
    return strdup(value);
}

char *Parser::read_config(FILE *fptr, char *keyword, char opera)
{
    char c;
    char tmp[512];
    char *value;
    int i = 0;
    escape_spaces(fptr);
    do
    {
        fread(&c, 1, 1, fptr);
	tmp[i] = c;
	i++;
    }
    while ((c != opera) && (c != '\n') && (!feof(fptr)));
    tmp[i] = '\0';
    //printf("%s\n",tmp);
    strcpy(keyword,trim(tmp));
    //printf("%s\n",keyword);
    if (c == '\n') return NULL; 
    escape_spaces(fptr);
    i = 0;
    do
    {
        fread(&c, 1, 1, fptr);
        tmp[i] = c;
        i++;
    }
    while ((c != '\n') && (!feof(fptr)));
    tmp[i] = '\0';
    value = trim(tmp);
    //printf("%s\n",value);
    //fscanf(fptr,"%s : %s\n",key,value);
    //printf("%s : %s\n",keyword,value);
    return value;
}


void Parser::get_line(FILE *fptr,char *buffer, int size)
{
    int i = 0;
    char c;
    do
    {
      fread(&c,1,1,fptr);
      buffer[i] = c;
      i++;
    }
    while ((!feof(fptr)) && (i < size) && (c != '\n'));
    buffer[i] = '\0';
}

char *Parser::get_token(const char *msg, char ascii, int num)
{
   char token[512];
   int i, ii = 0, count = 0;
   for (i = 0; (msg[i] != '\0') && (count < num); i++,ii++)
   {
	if (msg[i] == ascii) 
	{
		token[ii] = '\0';	
		ii = 0;
		count++;
	}				
	else token[ii] = msg[i];
   }
   return strdup(token);
} 


void Parser::insert_str(char *s, const char *k, const char *text)
{
   char result[512];
   int found;
   int i,ii, i1,i2 = 0;
   for (i = 0; s[i] != '\0'; i++,i2++)
   {
	found = 0;
	for (ii = 0; (k[ii] != '\0') && (s[i+ii] != '\0'); ii++)
	{
	  if (s[i+ii] == k[ii]) found++;
	}
	if (found == strlen(k))
	{
	   for (i1 = 0; text[i1] != '\0'; i1++,i2++)
		result[i2] = text[i1];
	   i2--;
	   i += strlen(k)-1;
	}
	else result[i2] = s[i];
   }  
   result[i2] = '\0';
   strcpy(s,result);
} 


int Parser::found_str(char **from, int max, char *s)
{
  int i;
  for (i = 0; i < max; i++)
  {
	if (strcmp(from[i],s) == 0) return i;
  }
  return -1; 
}


void Parser::clean_str(char *text)
{
  char tmp[512];
  int i = 0, ii, start, end;
  while (text[i] == ' ')
	i++;
  start = i;
  i = strlen(text)-1;
  while ((i > 0) && (text[i] == ' '))
  	i--;  
  end = i;
  for (i = start, ii = 0; i <= end; i++,ii++)
	tmp[ii] = text[i];
  tmp[ii] = '\0';
  strcpy(text,tmp);  
}

int Parser::is_numeric(const char *text)
{
  int i, ii;
  char DECNUM[] = {'0','1','2','3','4','5','6','7','8','9'};
  int found = 0;
  for (i = 0; text[i] != '\0'; i++)
  {
	for (ii = 0; ii < sizeof(DECNUM); ii++)
	if (text[i] == DECNUM[ii]) found++;
  }
  return (found == strlen(text));
}

